/**
 * easySchema - easyWSDL toolbox Platform.
 * Copyright (c) 2008,  eBM Websourcing
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.ow2.easywsdl.xml.util;

import java.io.StringWriter;
import java.util.Properties;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author alouis
 * @author Nicolas Salatge - eBM WebSourcing
 */
public class XMLUtil {
	
	private static final String transformerFactoryClass = "com.sun.org.apache.xalan.internal.xsltc.trax.TransformerFactoryImpl";
	
	  /**
     * <p>
     * A thread safe TransformerFactory, based on 
     * com.sun.org.apache.xalan.internal.xsltc.trax.TransformerFactoryImpl
     * </p>
     */
    public final static ThreadLocal<TransformerFactory> transformerFactoryThreadLocal = new ThreadLocal<TransformerFactory>() {

        @Override
        protected TransformerFactory initialValue() {
        	try {
        		Thread.currentThread().getContextClassLoader().loadClass(transformerFactoryClass);
				System.setProperty("javax.xml.transform.TransformerFactory", transformerFactoryClass);
	            } catch (ClassNotFoundException e) {
				System.err.println("Warning. EasyXML : TransformerFactory '"+transformerFactoryClass+"' not found, the standard TransformerFactory will be used.");
			}
        	    final TransformerFactory transformerFactory = TransformerFactory.newInstance();
                return transformerFactory;
        }
    };
	
    /**
     * <p>
     * This thread safe transformer has the following features:
     * <ul>
     * <li>omit the XML declaration,</li>
     * <li>default encoding</li>
     * </ul>
     * </p>
     */
    public final static ThreadLocal<Transformer> transformerWithoutXmlDeclarationThreadLocal = new ThreadLocal<Transformer>() {

        @Override
        protected Transformer initialValue() {
            try {

                final Transformer transformer = transformerFactoryThreadLocal.get().newTransformer();
                final Properties props = new Properties();
                props.put(OutputKeys.OMIT_XML_DECLARATION, "yes");
                props.put(OutputKeys.METHOD, "xml");
                transformer.setOutputProperties(props);
                return transformer;

            } catch (final TransformerConfigurationException e) {
                throw new RuntimeException("Failed to create Transformer", e);
            }
        }
    };

    /**
     * <p>
     * This thread safe transformer has the following features:
     * <ul>
     * <li>doesn't omit the XML declaration,</li>
     * <li>default encoding</li>
     * </ul>
     * </p>
     */
    public final static ThreadLocal<Transformer> defaultTransformerThreadLocal = new ThreadLocal<Transformer>() {

        @Override
        protected Transformer initialValue() {
            try {

                final Transformer transformer = transformerFactoryThreadLocal.get().newTransformer();
                final Properties props = new Properties();
                props.put(OutputKeys.OMIT_XML_DECLARATION, "no");
                props.put(OutputKeys.METHOD, "xml");
                transformer.setOutputProperties(props);
                return transformer;

            } catch (final TransformerConfigurationException e) {
                throw new RuntimeException("Failed to create Transformer", e);
            }
        }
    };
    
    /**
     * Create a String result from a DOM Node
     * Not encoding-safe !!
     * @param node
     *            the DOM Node. Must not be null
     * @return a String representation of the DOM Document
     * @throws TransformerException
     */
    public static String createStringFromDOMNode(final Node node) throws TransformerException {
        return XMLUtil.createStringFromDOMNode(node, true);
    }

    /**
     * Create a {@link String} from a Node list
     * Not encoding-safe !!
     * @param list
     * @return
     * @throws TransformerException
     */
    public static String createStringFromDOMNodeList(final NodeList list)
            throws TransformerException {
        final StringBuffer sb = new StringBuffer("");
        if (list != null) {
            for (int i = 0; i < list.getLength(); i++) {
                sb.append(XMLUtil.createStringFromDOMNode(list.item(i)));
            }
        }
        return sb.toString();
    }

    /**
     * Create a String result from a DOM Node
     * Not encoding-safe !!
     * @param node
     *            the DOM Node. Must not be null
     * @parma moitDeclaration If <code>true</code> no XML declaration will be
     *        added.
     * @return a String representation of the DOM Document
     * @throws TransformerException
     */
    public static String createStringFromDOMNode(final Node node, final boolean omitDeclaration)
            throws TransformerException {
        node.normalize();
        final Source source = new DOMSource(node);
        final StringWriter out = new StringWriter();
        final Result resultStream = new StreamResult(out);
        if (omitDeclaration) {
            XMLUtil.transformerWithoutXmlDeclarationThreadLocal.get().transform(source,
                    resultStream);
        } else {
            XMLUtil.defaultTransformerThreadLocal.get().transform(source, resultStream);
        }
        return out.toString();
    }

}
